<?php

namespace Drupal\smart_date;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Datetime\DrupalDateTime;

/**
 * Provides friendly methods for smart date range.
 */
trait SmartDateTrait {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    // retrieve settings from the display config
    $separator = $this->getSetting('separator');
    $join = $this->getSetting('join');
    $time_format = $this->getSetting('time_format');
    $date_format = $this->getSetting('date_format');
    $date_first = $this->getSetting('date_first');

    foreach ($items as $delta => $item) {
      if (!empty($item->value) && !empty($item->end_value)) {
        // apply date format from the display config
        $start_date = date($date_format, $item->value);
        $end_date = date($date_format, $item->end_value);
        // apply time format from the display config
        $start_time = date($time_format, $item->value);
        $end_time = date($time_format, $item->end_value);

        // Special formatting for ranges that span more than one day
        if ($start_date !== $end_date) {
          if ($date_first) {
            $elements[$delta] = [
              'start_time' => ['#plain_text' => $start_date . $join . $start_time],
              'separator' => ['#plain_text' => $separator],
              'end_time' => ['#plain_text' => $end_date . $join . $end_time],
            ];
          }
          else {
            $elements[$delta] = [
              'start_time' => ['#plain_text' => $start_time . $join . $start_date],
              'separator' => ['#plain_text' => $separator],
              'end_time' => ['#plain_text' => $end_time . $join . $end_date],
            ];
          }
        }
        else {
          // formatting for an all day event
          if (date('H:i', $item->value) == '00:00' && date('H:i', $item->end_value) == '23:59') {
            $time_output = $this->getSetting('allday_label');
          }
          else if ($item->value !== $item->end_value) {
            $time_output = $start_time . $separator . $end_time;
          }
          else {
            // simple formatting if no separate end time
            $time_output = $start_time;
          }
          if ($date_first) {
            $elements[$delta] = ['#plain_text' => $start_date . $join . $time_output];
          }
          else {
            $elements[$delta] = ['#plain_text' => $time_output . $join . $start_date];
          }
        }

        if (!empty($item->_attributes)) {
          $elements[$delta]['#attributes'] += $item->_attributes;
          // Unset field item attributes since they have been included in the
          // formatter output and should not be rendered in the field template.
          unset($item->_attributes);
        }
      }
    }

    return $elements;
  }
}
